-- Created by NuclearTonic
-- If reusing this code, please keep the name of the original author listed
-- in respect for borrowing said authors code.
-- RESPECT!

import "Tonic.TonicBars.Bar";

QuickslotBar = class( Tonic.TonicBars.Bar );

function QuickslotBar:Constructor( barid, sManager, bartype )
	Tonic.TonicBars.Bar.Constructor( self, barid, sManager, bartype );
	
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );
	if ( barSettings.barType ~= bartype ) then
		barSettings.barType = bartype;
		self.theSettingsManager:SetBarSettings( self.id, barSettings );
	end
	
	self:SetWantsKeyEvents( true );
	self.theplayer = Turbine.Gameplay.LocalPlayer();
	self.isVisible = true;
	self.effectsRegistered = false;
	self.combatRegistered = false;
	self.vitalsRegistered = false;
	
	if ( self.isModal ) then
		self.KeyDown = function( sender, args )
			if ( maitre_d:Alive( self.id ) == true ) then
				local changed = false;
				if ( args.Action == 268435556 ) then	-- ALT
					self.altIsDown = true;
					changed = true;
				end
				if ( args.Action == 268435575 ) then	-- CTRL
					self.controlIsDown = true;
					changed = true;
				end
				if ( args.Action == 268435458 ) then	-- SHIFT
					self.shiftIsDown = true;
					changed = true;
				end
				if ( args.Action == 268435635 ) then	-- F12
					self.isVisible = not self.isVisible;
					self:SetVisible( self.isVisible );
				end
				
				if ( changed ) then
					self:RefreshKeys();
				end
			end
		end

		self.KeyUp = function( sender, args )
			if ( maitre_d:Alive( self.id ) == true ) then
				local changed = false;
				if ( args.Action == 268435556 and self.altIsDown ~= nil ) then
					self.altIsDown = nil;
					changed = true;
				end
				if ( args.Action == 268435575 and self.controlIsDown ~= nil ) then
					self.controlIsDown = nil;
					changed = true;
				end
				if ( args.Action == 268435458 and self.shiftIsDown ~= nil ) then
					self.shiftIsDown = nil;
					changed = true;
				end
				
				if ( changed ) then
					self:RefreshKeys();
				end
			end
		end
	end

	self:UpdateBarExtensions();
end

function QuickslotBar:Create()
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );
	
	self.quickslotList = QuickslotList( self.id, self, self.theSettingsManager );
	self.quickslotList = self:GetQuickslotList()
	self.quickslotList:SetParent( self );
	self.quickslotList:SetMaxItemsPerLine( barSettings.quickslotColumns );

	self.qsCreated = true;
	self:Refresh();
	
	if ( self.isModal ) then
		self.quickslotTab = Turbine.UI.ListBox();
		self.quickslotTab:SetParent( self );
		self.quickslotTab:SetSize( self:GetWidth() * 0.5, 10 );
		self.quickslotTab:SetPosition( self:GetWidth() - self.quickslotTab:GetWidth(), 0 );
		self.quickslotTab:SetBackColor( Turbine.UI.Color(0,0,0,0)  );
		
		self.quickslotList:SetPosition(0, self.quickslotTab:GetHeight() );
		self:SetBarSize();

		self.quickslotTab.MouseDown = function( sender, args )
			self:MouseDown(args);
		end
		self.quickslotTab.MouseMove = function( sender, args )
			self:MouseMove(args);
		end
		self.quickslotTab.MouseUp = function( sender, args )
			self:MouseUp(args);
		end

	else
		self.quickslotList:SetPosition(0, 0 );
	end
end

function QuickslotBar:ClearTab( cTab )
	if ( self.quickslotTab == nil ) then
		return;
	end
	local settings = self.theSettingsManager:GetSettings();
	if ( settings.barMode == NORMAL_MODE ) then
		self.clearTab = cTab;
		self.quickslotTab:SetSize( self:GetWidth(), 10 );
	else
		self.clearTab = false;
		self.quickslotTab:SetSize( self:GetWidth() * 0.5, 10 );
	end
end

function Bar:SetBarSize()
	if ( self.quickslotTab ~= nil and self.isModal ) then
		self:SetSize( self.quickslotList:GetWidth(), self.quickslotList:GetHeight() + self.quickslotTab:GetHeight() );
	else
		self:SetSize( self.quickslotList:GetWidth(), self.quickslotList:GetHeight() );
	end
end

function QuickslotBar:RefreshKeys(refresh)
	if ( refresh == nil or refresh ) then
		self:Refresh();
	end
	local settings = self.theSettingsManager:GetSettings();
	if ( self.isModal and self.quickslotTab ~= nil ) then
		self.quickslotTab:SetBackColor( Turbine.UI.Color(0,0,0,0)  );
		if ( self.clearTab ~= nil and self.clearTab and settings.barMode == NORMAL_MODE ) then
			self.quickslotTab:SetSize( self:GetWidth(), 10 );
		else
			self.quickslotTab:SetSize( self:GetWidth() * 0.5, 10 );
		end
		self.quickslotTab:SetPosition( self:GetWidth() - self.quickslotTab:GetWidth(), 0 );
	end
	self:SetBarSize();
end

function QuickslotBar:Refresh()
	Tonic.TonicBars.Bar.Refresh( self );
	if ( self.quickslotTab == nil ) then
		return;
	end
	local settings = self.theSettingsManager:GetSettings();
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );

	if ( self.clearTab ~= nil and self.clearTab and settings.barMode == NORMAL_MODE ) then
		self.quickslotTab:SetSize( self:GetWidth(), 10 );
	else
		self.quickslotTab:SetSize( self:GetWidth() * 0.5, 10 );
	end
	self.quickslotTab:SetPosition( self:GetWidth() - self.quickslotTab:GetWidth(), 0 );
	
	if ( barSettings.displayOnPoison or barSettings.displayOnWound or barSettings.displayOnFear or barSettings.displayOnDisease ) then
		if ( not self.effectsRegistered ) then
			self:RegisterForEffectCallbacks();
		end
	else
		local effects = self.theplayer:GetEffects();
		effects.EffectAdded = nil;
		effects.EffectRemoved = nil;
		effects.EffectsCleared = nil;
		self.isPoisoned = nil;
		self.isWounded = nil;
		self.isFeared = nil;
		self.isDiseased = nil;
		self.effectsRegistered = false;
	end
	if ( barSettings.isCombatBar ) then
		if ( not self.combatRegistered ) then
			self:RegisterForCombatCallbacks();
		end
	else
		self.theplayer.InCombatChanged = nil;
		self.playerInCombat = nil;
		self.combatRegistered = false;
	end
	if ( barSettings.displayOnHealth ) then
		if ( not self.healthRegistered ) then
			self:RegisterForHealthCallbacks();
		end
	else
		self.theplayer.MoraleChanged = nil;
		self.healthTriggerActive = nil;
		self.healthRegistered = false;
	end
	if ( barSettings.displayOnPower ) then
		if ( not self.powerRegistered ) then
			self:RegisterForPowerCallbacks();
		end
	else
		self.theplayer.PowerChanged = nil;
		self.powerTriggerActive = nil;
		self.powerRegistered = false;
	end	
	self:DetermineVisiblity();
end

function QuickslotBar:RegisterForEffectCallbacks()
	self.effectsRegistered = true;
	self.isPoisoned = 0;
	self.isWounded = 0;
	self.isFeared = 0;
	self.isDiseased = 0;

    local effects = self.theplayer:GetEffects();
	for i = 1, effects:GetCount() do
		local effect = effects:Get(i);
		local category = effect:GetCategory();
		
		if(     category == Turbine.Gameplay.EffectCategory.Poison ) then
			self.isPoisoned = self.isPoisoned + 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Wound ) then
			self.isWounded = self.isWounded + 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Fear ) then
			self.isFeared = self.isFeared + 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Disease ) then
			self.isDiseased = self.isDiseased + 1;
		end
	end
	
	effects.EffectAdded = function(sender, args)
		local effects = self.theplayer:GetEffects()
		local effect = effects:Get(args.Index);
		local category = effect:GetCategory();
		
		effect:GetName();
		
		if(     category == Turbine.Gameplay.EffectCategory.Poison ) then
			self.isPoisoned = self.isPoisoned + 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Wound ) then
			self.isWounded = self.isWounded + 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Fear ) then
			self.isFeared = self.isFeared + 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Disease ) then
			self.isDiseased = self.isDiseased + 1;
		end
		self:DetermineVisiblity();
	end
	
	effects.EffectRemoved = function(sender, args)
		local effect = args.Effect;
		local category = effect:GetCategory();
		
		if(     category == Turbine.Gameplay.EffectCategory.Poison ) then
			self.isPoisoned = self.isPoisoned - 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Wound ) then
			self.isWounded = self.isWounded - 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Fear ) then
			self.isFeared = self.isFeared - 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Disease ) then
			self.isDiseased = self.isDiseased - 1;
		end
		self:DetermineVisiblity();
	end
	
	effects.EffectsCleared = function(sender, args)
		local effect = args.Effect;
		local category = effect:GetCategory();
		
		if(     category == Turbine.Gameplay.EffectCategory.Poison ) then
			self.isPoisoned = self.isPoisoned - 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Wound ) then
			self.isWounded = self.isWounded - 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Fear ) then
			self.isFeared = self.isFeared - 1;
		elseif( category == Turbine.Gameplay.EffectCategory.Disease ) then
			self.isDiseased = self.isDiseased - 1;
		end
		self:DetermineVisiblity();
	end
end

function QuickslotBar:RegisterForCombatCallbacks()
	self.combatRegistered = true;
	self.theplayer.InCombatChanged = function( sender, args )
		if ( self.theplayer:IsInCombat() ) then
			self.playerInCombat = true;
		else
			self.playerInCombat = nil;
		end
		self:DetermineVisiblity();
	end
end

function QuickslotBar:RegisterForHealthCallbacks()
	self.healthRegistered = true;
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );
	self.theplayer.MoraleChanged = function( sender, args )
		local limit = barSettings.healthTrigger;
		local value = self.theplayer:GetMorale() / self.theplayer:GetMaxMorale();
		if ( value <= limit ) then
			self.healthTriggerActive = true;
		else
			self.healthTriggerActive = nil;
		end
		self:DetermineVisiblity();
	end
end
	
function QuickslotBar:RegisterForPowerCallbacks()
	self.powerRegistered = true;
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );
	self.theplayer.PowerChanged = function( sender, args )
		local limit = barSettings.powerTrigger;
		local value = self.theplayer:GetPower() / self.theplayer:GetMaxPower();
		if ( value <= limit ) then
			self.powerTriggerActive = true;
		else
			self.powerTriggerActive = nil;
		end
		self:DetermineVisiblity();
	end
end

function QuickslotBar:DetermineVisiblity()
	local settings = self.theSettingsManager:GetSettings();
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );

	if ( settings.barMode == NORMAL_MODE and barSettings.barType == QUICKSLOTBAR ) then
		local visible = false;
		if ( barSettings.isAlt and self.altIsDown ~= nil ) then	
			visible = true;
		end
		if ( barSettings.isControl and self.controlIsDown ~= nil ) then	
			visible = true;
		end
		if ( barSettings.isShift and self.shiftIsDown ~= nil ) then	
			visible = true;
		end
		if ( barSettings.isCombatBar and self.playerInCombat ~= nil ) then
			visible = true;
		end

		if ( barSettings.displayOnPoison and self.isPoisoned > 0 ) then
			visible = true;
		end
		if ( barSettings.displayOnWound and self.isWounded > 0 ) then
			visible = true;
		end
		if ( barSettings.displayOnFear and self.isFeared > 0 ) then
			visible = true;
		end
		if ( barSettings.displayOnDisease and self.isDiseased > 0 ) then
			visible = true;
		end
		
		if ( barSettings.displayOnHealth and self.healthTriggerActive ~= nil ) then
			visible = true;
		end
		if ( barSettings.displayOnPower and self.powerTriggerActive ~= nil ) then
			visible = true;
		end
		
		if ( visible == false and self.isVisible == true ) then
			visible = barSettings.visible;
		end
	
		self:SetVisible( visible );
	end
end

function Bar:ClearQuickslots( removed )
	self.quickslotList:ClearQuickslots();
	if ( self.extensionBars ~= nil and removed ~= nil ) then
		for key, value in pairs (self.extensionBars) do
			maitre_d:Remove( self.extensionBars[ key ].bar:GetID() );
			self.extensionBars[ key ].bar = nil;
			self.extensionBars[ key ] = nil;
		end
	end
end

function QuickslotBar:UpdateBarExtensions()
	local barSettings = self.theSettingsManager:GetBarSettings( self.id );
	if ( self.extensionBars == nil ) then
		return;
	end

	for key, value in pairs ( self.extensionBars ) do
		local x, y = self.quickslotList:GetQuickslotLocation( value.quickslot );
		self.extensionBars[ key ].bar:Setup( x, y );
	end
end

function QuickslotBar:RegisterBarExtension( extBar, index, extensionBarID )
	local attachedbarSettings = self.theSettingsManager:GetBarSettings( extensionBarID );
	if ( attachedbarSettings.connectionQuickslotID ~= index or attachedbarSettings.connectionBarID ~= self.id ) then
		attachedbarSettings.connectionQuickslotID = index;
		attachedbarSettings.connectionBarID = self.id;
		self.theSettingsManager:SetBarSettings( extensionBarID, attachedbarSettings );
	end

	local barSettings = self.theSettingsManager:GetBarSettings( self.id );
	if ( self.nextExtension == nil ) then
		self.nextExtension = 1;
	end
	
	if ( barSettings.barType == QUICKSLOTBAR ) then
		if ( self.extensionBars == nil ) then
			self.extensionBars = {};
		end
		self.extensionBars[ self.nextExtension ] = {};
		self.extensionBars[ self.nextExtension ].bar = extBar;
		self.extensionBars[ self.nextExtension ].quickslot = index;
		self.extensionBars[ self.nextExtension ].cycleCount = 0;
		self.nextExtension = self.nextExtension + 1;
		
		self.quickslotList:SetupExtensionSlot( self.extensionBars, index );
	else
		Turbine.Shell.WriteLine( "You can not add extensions to an extension slot. Please choose a green bar." );
	end
end